@@ -12,7 +12,7 @@ export PS4='+ ${BASH_SOURCE:-}:${FUNCNAME[0]:-}:L${LINENO:-}:   '
 # Keep track of the DevStack directory
 TOP_DIR=$(cd $(dirname "$0")/.. && pwd)
 
-while getopts "n:c:i:m:M:d:a:b:e:E:p:o:f:l:L:N:A:D:v:P:" arg; do
+while getopts "n:c:i:m:M:d:a:b:e:E:p:o:f:l:L:N:A:D:v:P:t:B:" arg; do
     case $arg in
         n) NAME=$OPTARG;;
         c) CPU=$OPTARG;;
@@ -36,6 +36,8 @@ while getopts "n:c:i:m:M:d:a:b:e:E:p:o:f:l:L:N:A:D:v:P:" arg; do
         D) NIC_DRIVER=$OPTARG;;
         v) VOLUME_COUNT=$OPTARG;;
         P) STORAGE_POOL=$OPTARG;;
+        t) MACHINE_TYPE=$OPTARG;;
+        B) BLOCK_SIZE=$OPTARG;;
     esac
 done
 
@@ -76,6 +78,8 @@ if [ ! -z "$UEFI_LOADER" ]; then
     fi
 fi
 
+BLOCK_SIZE=${BLOCK_SIZE:-512}
+
 # Create bridge and add VM interface to it.
 # Additional interface will be added to this bridge and
 # it will be plugged to OVS.
@@ -123,11 +127,13 @@ if ! virsh list --all | grep -q $NAME; then
     if [[ -n "$EMULATOR" ]]; then
         vm_opts+="--emulator $EMULATOR "
     fi
+
     $PYTHON $TOP_DIR/scripts/configure-vm.py \
         --bootdev network --name $NAME \
         --arch $ARCH --cpus $CPU --memory $MEM --libvirt-nic-driver $LIBVIRT_NIC_DRIVER \
         --disk-format $DISK_FORMAT $VM_LOGGING --engine $ENGINE $UEFI_OPTS $vm_opts \
-        --interface-count $INTERFACE_COUNT $MAC_ADDRESS >&2
+        --interface-count $INTERFACE_COUNT $MAC_ADDRESS --machine_type $MACHINE_TYPE \
+        --block-size $BLOCK_SIZE >&2
 fi
 
 # echo mac in format mac1,ovs-node-0i1;mac2,ovs-node-0i2;...;macN,ovs-node0iN
