@@ -13,7 +13,6 @@
 #    under the License.
 
 import contextlib
-import crypt
 import datetime
 import functools
 import os
@@ -26,6 +25,7 @@ from oslo_log import log
 from oslo_serialization import jsonutils
 from oslo_service import loopingcall
 from oslo_utils import excutils
+from oslo_utils import secretutils
 from oslo_utils import strutils
 from oslo_utils import timeutils
 
@@ -49,8 +49,8 @@ CONF = cfg.CONF
 
 
 PASSWORD_HASH_FORMAT = {
-    'sha256': crypt.METHOD_SHA256,
-    'sha512': crypt.METHOD_SHA512,
+    'sha256': 'SHA-256',
+    'sha512': 'SHA-512',
 }
 
 
@@ -290,7 +290,12 @@ def node_power_action(task, new_state, timeout=None):
              storage interface upon setting power on.
     :raises: other exceptions by the node's power driver if something
              wrong occurred during the power action.
-
+    :raises: ChildNodeLocked when a child node must be acted upon due
+             to this request, but we cannot act upon it due to a lock
+             being held.
+    :raises: ParentNodeLocked when a parent node must be acted upon
+             due to this request, but we cannot act upon it due to a
+             lock being held.
     """
     notify_utils.emit_power_set_notification(
         task, fields.NotificationLevel.INFO, fields.NotificationStatus.START,
@@ -313,6 +318,15 @@ def node_power_action(task, new_state, timeout=None):
             node.save()
         return
 
+    # Parent node power required?
+    if task.node.parent_node:
+        # If we have a parent node defined for the device, we likely
+        # need to turn the power on.
+        parent_power_needed = not task.node.driver_info.get(
+            'has_dedicated_power_supply', False)
+    else:
+        parent_power_needed = False
+
     # Set the target_power_state and clear any last_error, if we're
     # starting a new operation. This will expose to other processes
     # and clients that work is in progress. Keep the last_error intact
@@ -336,6 +350,12 @@ def node_power_action(task, new_state, timeout=None):
             task.driver.storage.attach_volumes(task)
 
         if new_state != states.REBOOT:
+            if new_state == states.POWER_ON and parent_power_needed:
+                _handle_child_power_on(task, target_state, timeout)
+            if new_state == states.POWER_OFF:
+                _handle_child_power_off(task, target_state, timeout)
+            # Parent/Child dependencies handled, we can take care of
+            # the original node now.
             task.driver.power.set_power_state(task, new_state, timeout=timeout)
         else:
             # TODO(TheJulia): We likely ought to consider toggling
@@ -384,6 +404,68 @@ def node_power_action(task, new_state, timeout=None):
                             {'node': node.uuid, 'error': e})
 
 
+def _handle_child_power_on(task, target_state, timeout):
+    """Actions related to powering on a parent for a child node."""
+    with task_manager.acquire(
+            task.context, task.node.parent_node,
+            shared=True,
+            purpose='power on parent node') as pn_task:
+        if (pn_task.driver.power.get_power_state(pn_task)
+                != states.POWER_ON):
+            try:
+                pn_task.upgrade_lock()
+            except exception.NodeLocked as e:
+                LOG.error('Cannot power on parent_node %(pn)s '
+                          'as it is locked by %(host)s. We are '
+                          'unable to proceed with the current '
+                          'operation for child node %(node)s. '
+                          'Error: %(error)s',
+                          {'pn': pn_task.node.uuid,
+                           'host': pn_task.node.reservation,
+                           'node': task.node.uuid,
+                           'error': e})
+                raise exception.ParentNodeLocked(
+                    node=task.node.uuid, parent=pn_task.node.uuid)
+            node_power_action(pn_task, target_state, timeout)
+
+
+def _handle_child_power_off(task, target_state, timeout):
+    """Actions related to powering off child nodes."""
+    child_nodes = task.node.list_child_node_ids(exclude_dedicated_power=True)
+    cn_tasks = []
+    try:
+        for child in child_nodes:
+            # Get the task for the node, check the status, add
+            # the list of locks to act upon.
+            cn_task = task_manager.acquire(
+                task.context, child, shared=True,
+                purpose='child node power off')
+            if (cn_task.driver.power.get_power_state(cn_task)
+                != states.POWER_OFF):
+                cn_task.upgrade_lock()
+                cn_tasks.append(cn_task)
+        for cn_task in cn_tasks:
+            node_power_action(cn_task, target_state, timeout)
+    except exception.NodeLocked as e:
+        LOG.error(
+            'Cannot power off child node %(cn)s '
+            'as it is locked by %(host)s. We are '
+            'unable to proceed with the current '
+            'operation for parent node %(node)s. '
+            'Error: %(error)s',
+            {'cn': cn_task.node.uuid,
+             'host': cn_task.node.reservation,
+             'node': task.node.uuid,
+             'error': e})
+        raise exception.ChildNodeLocked(
+            node=task.node.uuid, child=cn_task.node.uuid)
+    finally:
+        for cn_task in cn_tasks:
+            # We don't need to do anything else with the child node,
+            # and we need to release the child task's lock.
+            cn_task.release_resources()
+
+
 @task_manager.require_exclusive_lock
 def cleanup_after_timeout(task):
     """Cleanup deploy task after timeout.
@@ -1393,7 +1475,7 @@ def make_salt():
 
     :returns: a valid salt for use with crypt.crypt
     """
-    return crypt.mksalt(
+    return secretutils.crypt_mksalt(
         method=PASSWORD_HASH_FORMAT[
             CONF.conductor.rescue_password_hash_algorithm])
 
@@ -1403,7 +1485,7 @@ def hash_password(password=''):
 
     :param password: password to be hashed
     """
-    return crypt.crypt(password, make_salt())
+    return secretutils.crypt_password(password, make_salt())
 
 
 def get_attached_vif(port):
