@@ -30,6 +30,7 @@ from ironic.conf import dhcp
 from ironic.conf import disk_utils
 from ironic.conf import dnsmasq
 from ironic.conf import drac
+from ironic.conf import exception
 from ironic.conf import fake
 from ironic.conf import glance
 from ironic.conf import healthcheck
@@ -38,8 +39,9 @@ from ironic.conf import inspector
 from ironic.conf import inventory
 from ironic.conf import ipmi
 from ironic.conf import irmc
+from ironic.conf import json_rpc
+from ironic.conf import mdns
 from ironic.conf import metrics
-from ironic.conf import metrics_statsd
 from ironic.conf import molds
 from ironic.conf import neutron
 from ironic.conf import nova
@@ -67,6 +69,7 @@ drac.register_opts(CONF)
 dhcp.register_opts(CONF)
 disk_utils.register_opts(CONF)
 dnsmasq.register_opts(CONF)
+exception.register_opts(CONF)
 fake.register_opts(CONF)
 glance.register_opts(CONF)
 healthcheck.register_opts(CONF)
@@ -75,8 +78,9 @@ inspector.register_opts(CONF)
 inventory.register_opts(CONF)
 ipmi.register_opts(CONF)
 irmc.register_opts(CONF)
+json_rpc.register_opts(CONF)
+mdns.register_opts(CONF)
 metrics.register_opts(CONF)
-metrics_statsd.register_opts(CONF)
 molds.register_opts(CONF)
 neutron.register_opts(CONF)
 nova.register_opts(CONF)
